@@ -1,6 +1,5 @@
 from __future__ import print_function
 
-import tnorm.constants
 import tkinter as tk
 
 from tkinter import ttk
@@ -13,16 +12,16 @@ def generate_summary(tnorm_app):
         print('Generating summary page... ', end='')
         W=tnorm_app.wrapper
         B=tnorm_app.ball
-        P=tnorm_app.ball.polyhedron
-        M=tnorm_app.wrapper.manifold
+        P=tnorm_app.ball.polyhedron()
+        M=tnorm_app.wrapper.manifold()
         try:
             sym_group = M.symmetry_group()
         except ValueError:
             sym_group = 'Failed'
         sections = ['Manifold', 'Thurston norm ball', 'Normal surfaces']
         names = [('rank H_2(M;bdy M)', 'volume', 'num tetrahedra', 'num cusps', 'symmetry group'),('boundary H1 basis', 'num vertices', 'vertex degrees', 'vertices qtons rep (genus,punctures)', 'num faces', 'top dim faces (<verts>)'),('allows non-admissible', 'num qtons', 'num non-trivial in H2', 'num norm minimizing')]
-        qtons_inds = [v.surface_index for v in B.vertices]
-        values = [(M.homology().rank(), M.volume(), M.num_tetrahedra(), M.num_cusps(), sym_group), (W.bdy_H1_basis, len(P.vertices()), [sum(P.vertex_adjacency_matrix()[i]) for i in range(P.n_vertices())], [(v.genus,v.num_boundary_comps) for v in B.vertices], len(P.faces(P.dim()-1)), ['<{}>'.format(' '.join([str(v.index()) for v in face.vertices()])) for face in P.faces(P.dim()-1)]), (W.allows_non_admissible, W.qtons().size(), len([1 for i in range(W.qtons().size()) if W.over_facet(i)!=None]), len([1 for i in range(W.qtons().size()) if W.is_norm_minimizing(i)]))]
+        qtons_inds = [v.qtons_index() for v in B.vertices()]
+        values = [(M.homology().rank(), M.volume(), M.num_tetrahedra(), M.num_cusps(), sym_group), (W.bdy_H1_basis(), len(P.vertices()), [sum(P.vertex_adjacency_matrix()[i]) for i in range(P.n_vertices())], [(v.genus(),v.num_boundary_comps()) for v in B.vertices()], len(P.faces(P.dim()-1)), ['<{}>'.format(' '.join([str(v.index()) for v in face.vertices()])) for face in P.faces(P.dim()-1)]), (W.allows_non_admissible(), W.qtons().size(), len([1 for i in range(W.qtons().size()) if W.over_facet(i)!=None]), len([1 for i in range(W.qtons().size()) if W.is_norm_minimizing(i)]))]
         summary_page(tnorm_app.SummaryTab, sections, names, values)
         print('Done.')
     except Exception as e: print('Error: {}'.format(e))
@@ -32,12 +31,12 @@ def generate_nb_overview(tnorm_app):
     try:
         W=tnorm_app.wrapper
         B=tnorm_app.ball
-        P=tnorm_app.ball.polyhedron
-        M=tnorm_app.wrapper.manifold
+        P=tnorm_app.ball.polyhedron()
+        M=tnorm_app.wrapper.manifold()
 
         sections = ['Overview']
         names = [('dimension', 'is compact?', 'num vertices', 'num rays', 'vertex degrees', 'vertices qtons rep (genus,punctures)', 'all vertices admissible?', 'num faces', 'faces (<vertices by index>)')]
-        values = [(B.dimension, B.is_compact, len(P.vertices()), B.num_rays, [sum(P.vertex_adjacency_matrix()[i]) for i in range(P.n_vertices())], [(v.genus,v.num_boundary_comps) for v in B.vertices], B.all_vertices_admissible, len(P.faces(P.dim()-1)), ['<{}>'.format(' '.join([str(B.index_of_poly_vert(v)) for v in face.vertices()])) for face in P.faces(P.dim()-1)])]
+        values = [(B.dimension(), B.is_compact(), len(P.vertices()), B.num_rays(), [sum(P.vertex_adjacency_matrix()[i]) for i in range(P.n_vertices())], [(v.genus(),v.num_boundary_comps()) for v in B.vertices()], B.all_vertices_admissible(), len(P.faces(P.dim()-1)), ['<{}>'.format(' '.join([str(B.index_of_poly_vert(v)) for v in face.vertices()])) for face in P.faces(P.dim()-1)])]
         summary_page(tnorm_app.NormBallTab, sections, names, values)
     except Exception as e: print('Error: {}'.format(e))
     tnorm_app.stop_spin()
@@ -46,12 +45,12 @@ def generate_dnb_overview(tnorm_app):
     try:
         W=tnorm_app.wrapper
         B=tnorm_app.dual_ball
-        P=tnorm_app.dual_ball.polyhedron
-        M=tnorm_app.wrapper.manifold
+        P=tnorm_app.dual_ball.polyhedron()
+        M=tnorm_app.wrapper.manifold()
 
         sections = ['Overview']
         names = [('dimension', 'is full dimensional?', 'num vertices', 'vertex degrees', 'num faces', 'faces (<vertices by index>)')]
-        values = [(B.dimension, B.is_full_dimensional, len(P.vertices()), [sum(P.vertex_adjacency_matrix()[i]) for i in range(P.n_vertices())], len(P.faces(P.dim()-1)), ['<{}>'.format(' '.join([str(B.index_of_poly_vert(v)) for v in face.vertices()])) for face in P.faces(P.dim()-1)])]
+        values = [(B.dimension(), B.is_full_dimensional(), len(P.vertices()), [sum(P.vertex_adjacency_matrix()[i]) for i in range(P.n_vertices())], len(P.faces(P.dim()-1)), ['<{}>'.format(' '.join([str(B.index_of_poly_vert(v)) for v in face.vertices()])) for face in P.faces(P.dim()-1)])]
         summary_page(tnorm_app.DualNormBallTab, sections, names, values)
     except Exception as e: print('Error: {}'.format(e))
     tnorm_app.stop_spin()
