@@ -1,11 +1,13 @@
 
 from tnorm.version import version, __version__
-import sys
-
-
-#import tnorm.kernel
-#import tnorm.GUI
 
 from tnorm.TN_wrapper import TN_wrapper as load
 
+import tnorm.kernel
+import tnorm.extras
+import tnorm.utilities
+from tnorm.test import test
+#import tnorm.GUI
+
+
 #IN_SAGE=False
